// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//
void lcd_data(unsigned char temp1);
void lcd_string(char *data);
void lcd_command(unsigned char temp1);
void lcd_enable(void);
void lcd_init(void);
void lcd_home(void);
void lcd_clear(void);
void set_cursor(uint8_t x, uint8_t y);
 
// LCD Befehle
 
#define CLEAR_DISPLAY 0x01
#define CURSOR_HOME   0x02
 
// Pinbelegung fr das LCD, an verwendete Pins anpassen
 
#define LCD_PORT      PORTB
#define LCD_DDR       DDRB
#define LCD_RS        PB4
#define LCD_EN        PB5
#define LCD_PORT2	  PORTC
#define LCD_DDR2	  DDRC
#define LCD_ON_OFF	  PC3
// DB4 bis DB7 des LCD sind mit PB0 bis PB3 des AVR verbunden 


// Globale Variabeln
uint8_t lcd_x = 0, lcd_y = 1;

// sendet ein Datenbyte an das LCD
 
void lcd_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// sendet einen Befehl an das LCD
 
void lcd_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp1;               // setzen
   lcd_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_PORT &= 0xF0;
   LCD_PORT |= temp2;               // setzen
   lcd_enable();
   
   _delay_us(42);
}
 
// erzeugt den Enable-Puls
void lcd_enable(void)
{
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
   // http://www.mikrocontroller.net/topic/81974#685882
   LCD_PORT |= (1<<LCD_EN);
    _delay_us(5);                   // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
   LCD_PORT &= ~(1<<LCD_EN);
}
 

// Steuerung der Displaybeleuchtung
// i = 1 :Display an
// i = 0 :Display aus
void lcd_bg(uint8_t i){
	
	if( i == 1 ) LCD_PORT2 &= ~(1<<LCD_ON_OFF);  // LCD-Display an
	else LCD_PORT2 |= (1<<LCD_ON_OFF);			 // LCD-Display aus

}


// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd_init(void)
{
   LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) | (1<<LCD_EN);   // Port1 auf Ausgang schalten
   LCD_DDR2 = LCD_DDR2 | (1<<LCD_ON_OFF);				   // Port2,Pin On/Off auf Ausgang schalten

   lcd_bg(1);
 
   // muss 3mal hintereinander gesendet werden zur Initialisierung
 
   _delay_ms(15);
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x03;            
   LCD_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd_enable();
 
   _delay_ms(5);
   lcd_enable();
 
   _delay_ms(1);
   lcd_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_PORT &= 0xF0;
   LCD_PORT |= 0x02;
   lcd_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x7
   lcd_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd_command(0x06);
 
   lcd_clear();
}
 
// Sendet den Befehl zur Lschung des Displays
 
void lcd_clear(void)
{
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(10);
}
 
// Sendet den Befehl: Cursor Home
 
void lcd_home(void)
{
   	lcd_command(CURSOR_HOME);

  	lcd_x = 0;
  	lcd_y = 1;

   	_delay_ms(10);
}
 
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
 
void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }

  lcd_command(tmp);

  lcd_x = x;
  lcd_y = y;

}

// Schreibt eine Statusnachricht auf's Display
void lcd_write_status(char *data)
{
	
	lcd_clear();
	lcd_home();

	while(*data){
		lcd_data(*data);
		data++;
	}
}
 
// Schreibt einen String auf das LCD 
void lcd_string(char *data)
{

	if( lcd_x == 8 && lcd_y == 1 ){

		set_cursor(0, 2);				// Display auf zweite Hlfte schalten
		lcd_x = 0;
		lcd_y = 2;		

	}

	if( lcd_x == 8 && lcd_y == 2 ){
		
		lcd_clear();					// Display lschen
		lcd_home();						// Display wieder auf Anfang setzen

	}


    while(*data) {

        lcd_data(*data);
        data++;

		lcd_x++;

    }
}
